@@ -1,4 +1,4 @@
-/* $Id: 32bpp_anim.hpp 23729 2012-01-03 10:50:06Z truebrain $ */
+/* $Id: 32bpp_anim.hpp 27863 2017-05-03 20:09:51Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -15,19 +15,25 @@
 #include "32bpp_optimized.hpp"
 
 /** The optimised 32 bpp blitter with palette animation. */
-class Blitter_32bppAnim FINAL : public Blitter_32bppOptimized {
-private:
+class Blitter_32bppAnim : public Blitter_32bppOptimized {
+protected:
 	uint16 *anim_buf;    ///< In this buffer we keep track of the 8bpp indexes so we can do palette animation
 	int anim_buf_width;  ///< The width of the animation buffer.
 	int anim_buf_height; ///< The height of the animation buffer.
+	int anim_buf_pitch;  ///< The pitch of the animation buffer.
 	Palette palette;     ///< The current palette.
 
 public:
 	Blitter_32bppAnim() :
 		anim_buf(NULL),
 		anim_buf_width(0),
-		anim_buf_height(0)
-	{}
+		anim_buf_height(0),
+		anim_buf_pitch(0)
+	{
+		this->palette = _cur_palette;
+	}
+
+	~Blitter_32bppAnim();
 
 	/* virtual */ void Draw(Blitter::BlitterParams *bp, BlitterMode mode, ZoomLevel zoom);
 	/* virtual */ void DrawColourMappingRect(void *dst, int width, int height, PaletteID pal);
@@ -47,19 +53,18 @@ public:
 	/**
 	 * Look up the colour in the current palette.
 	 */
-	inline uint32 LookupColourInPalette(uint index)
+	inline Colour LookupColourInPalette(uint index)
 	{
-		return this->palette.palette[index].data;
+		return this->palette.palette[index];
 	}
 
 	template <BlitterMode mode> void Draw(const Blitter::BlitterParams *bp, ZoomLevel zoom);
 };
 
 /** Factory for the 32bpp blitter with animation. */
-class FBlitter_32bppAnim: public BlitterFactory<FBlitter_32bppAnim> {
+class FBlitter_32bppAnim : public BlitterFactory {
 public:
-	/* virtual */ const char *GetName() { return "32bpp-anim"; }
-	/* virtual */ const char *GetDescription() { return "32bpp Animation Blitter (palette animation)"; }
+	FBlitter_32bppAnim() : BlitterFactory("32bpp-anim", "32bpp Animation Blitter (palette animation)") {}
 	/* virtual */ Blitter *CreateInstance() { return new Blitter_32bppAnim(); }
 };
 
