@@ -1,4 +1,4 @@
-/* $Id: strings_func.h 24607 2012-10-17 19:15:23Z rubidium $ */
+/* $Id: strings_func.h 27758 2017-02-26 19:41:30Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -15,6 +15,49 @@
 #include "strings_type.h"
 #include "string_type.h"
 #include "gfx_type.h"
+#include "core/bitmath_func.hpp"
+
+/**
+ * Extract the StringTab from a StringID.
+ * @param str String identifier
+ * @return StringTab from \a str
+ */
+static inline StringTab GetStringTab(StringID str)
+{
+	StringTab result = (StringTab)(str >> TAB_SIZE_BITS);
+	if (result >= TEXT_TAB_NEWGRF_START) return TEXT_TAB_NEWGRF_START;
+	if (result >= TEXT_TAB_GAMESCRIPT_START) return TEXT_TAB_GAMESCRIPT_START;
+	return result;
+}
+
+/**
+ * Extract the StringIndex from a StringID.
+ * @param str String identifier
+ * @return StringIndex from \a str
+ */
+static inline uint GetStringIndex(StringID str)
+{
+	return str - (GetStringTab(str) << TAB_SIZE_BITS);
+}
+
+/**
+ * Create a StringID
+ * @param tab StringTab
+ * @param index StringIndex
+ * @return StringID composed from \a tab and \a index
+ */
+static inline StringID MakeStringID(StringTab tab, uint index)
+{
+	if (tab == TEXT_TAB_NEWGRF_START) {
+		assert(index < TAB_SIZE_NEWGRF);
+	} else if (tab == TEXT_TAB_GAMESCRIPT_START) {
+		assert(index < TAB_SIZE_GAMESCRIPT);
+	} else {
+		assert(tab < TEXT_TAB_END);
+		assert(index < TAB_SIZE);
+	}
+	return (tab << TAB_SIZE_BITS) + index;
+}
 
 class StringParameters {
 	StringParameters *parent; ///< If not NULL, this instance references data from this parent instance.
@@ -56,7 +99,7 @@ public:
 		offset(0),
 		num_param(size)
 	{
-		assert(size <= parent.num_param - parent.offset);
+		assert(size <= parent.GetDataLeft());
 		if (parent.type == NULL) {
 			this->type = NULL;
 		} else {
@@ -89,6 +132,12 @@ public:
 		return &this->data[this->offset];
 	}
 
+	/** Return the amount of elements which can still be read. */
+	uint GetDataLeft() const
+	{
+		return this->num_param - this->offset;
+	}
+
 	/** Get a pointer to a specific element in the data array. */
 	uint64 *GetPointerToOffset(uint offset) const
 	{
@@ -124,7 +173,6 @@ public:
 };
 extern StringParameters _global_string_params;
 
-char *InlineString(char *buf, StringID string);
 char *GetString(char *buffr, StringID string, const char *last);
 char *GetStringWithArgs(char *buffr, StringID string, StringParameters *args, const char *last, uint case_index = 0, bool game_script = false);
 const char *GetStringPtr(StringID string);
@@ -155,6 +203,9 @@ static inline void SetDParam(uint n, uint64 v)
 	_global_string_params.SetParam(n, v);
 }
 
+void SetDParamMaxValue(uint n, uint64 max_value, uint min_count = 0, FontSize size = FS_NORMAL);
+void SetDParamMaxDigits(uint n, uint count, FontSize size = FS_NORMAL);
+
 void SetDParamStr(uint n, const char *str);
 
 void CopyInDParam(int offs, const uint64 *src, int num);
